/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __ls10_dev_perf_h__
#define __ls10_dev_perf_h__
/* This file is autogenerated.  Do not edit */
#define NV_PERF_PMM_PERDOMAIN_OFFSET                                                        0x00000200 /*       */
#define NV_PERF_PMM                                                              0x000003FF:0x00000000 /* RW--D */
#define NV_PERF_PMMROUTER                                                        0x000001FF:0x00000000 /* RW--D */
#define NV_PERF_PMMROUTER_CG2                                                                              0x00000040 /* RW-4R */
#define NV_PERF_PMMROUTER_CG2_SLCG                                                                              31:31 /* RWEVF */
#define NV_PERF_PMMROUTER_CG2_SLCG_ENABLED                                                                 0x00000000 /* RWE-V */
#define NV_PERF_PMMROUTER_CG2_SLCG_DISABLED                                                                0x00000001 /* RW--V */
#define NV_PERF_PMMROUTER_CG1_SECURE                                                                                     0x00000054 /* RW-4R */
#define NV_PERF_PMMROUTER_CG1_SECURE_FLCG_PERFMON                                                                             31:31 /* RWEUF */
#define NV_PERF_PMMROUTER_CG1_SECURE_FLCG_PERFMON_DISABLED                                                               0x00000000 /* RWE-V */
#define NV_PERF_PMMROUTER_CG1_SECURE_FLCG_PERFMON__PROD                                                                  0x00000001 /* RW--V */
#define NV_PERF_PMMROUTER_CG1_SECURE_FLCG_PERFMON_ENABLED                                                                0x00000001 /* RW--V */
#define NV_PERF_PMMROUTER_PERFMON_CG2_SECURE                                                                           0x00000058 /* RW-4R */
#define NV_PERF_PMMROUTER_PERFMON_CG2_SECURE_SLCG                                                                             31:31 /* RWEUF */
#define NV_PERF_PMMROUTER_PERFMON_CG2_SECURE_SLCG_ENABLED                                                                0x00000000 /* RWE-V */
#define NV_PERF_PMMROUTER_PERFMON_CG2_SECURE_SLCG__PROD                                                                  0x00000000 /* RW--V */
#define NV_PERF_PMMROUTER_PERFMON_CG2_SECURE_SLCG_DISABLED                                                               0x00000001 /* RW--V */
#endif // __ls10_dev_perf_h__
